/*
 * RenderFTP.jsx
 * Author:	Peter Torpey
 *        	www.petertorpey.com
 * Version:	1.0	01-21-05
 *         	Original code Copyright 2005 Peter Torpey.
 *         	This code may be used, distributed, and modified for personal use
 *         	provided the name and URL of the original author, above, remain
 *         	intact.
 *
 * 			If this code is used in the creation of a notable production,
 * 			please let the author know. The author's e-mail address may be
 * 			found at http://www.petertorpey.com/contact.html .
 *
 *         	Portions of this file are based loosely on render_and_email.jsx
 *         	from Adobe Systems Inc.
 *
 * Dependencies:
 *  (lib)/FTPConnection.jsx
 *  (lib)/UILayout_13.jsx
 *
 * Description:
 * 	Render queue items are rendered and uploaded to an FTP server.
 *
 * Usage:
 *  Run this script to render currently enqueued items and have them uploaded
 *  	to an FTP server.
 *  You will be prompted for information necessary to connect to the server.
 *  	This information may be stored for subsequent sessions.
 *  Each output file of the rendered items will be uploaded to the specified
 *  	directory on the server and have the same name as the local file.
 *
 * Notes:
 *  This script and the scripts upon which it depends have been tested under
 *  	AE6.5 on WinXP.
 *
 * Known Issues:
 *  Files are not uploaded until all enqueued items are rendered. This could be
 *  	changed so that files are uploaded as soon as each has been rendered.
 *  	Is this really desirable?
 *
 * For the Future:
 *  Encrypt saved password. (Yes, even though it is sent as plaintext.)
 * 	Support for rendered image sequences.
 *
 */

{
	var NAME_STR = "Render and FTP";
	var readyState = true;

	var FTP_SETTINGS_CAT = "FTP Settings";
	var FTP_SETTINGS_HOST = "Host Address";
	var FTP_SETTINGS_UNAME = "User Name";
	var FTP_SETTINGS_UPASS = "Password";
	var FTP_SETTINGS_ANON = "Anonymous Login";
	var FTP_SETTINGS_PATH = "Remote Path";

	var FTP_DFLT_PORT = 21;

	// includes
	var includeScripts = ["(lib)/UILayout_13.jsx", "(lib)/FTPConnection.jsx"];
	for (var i = 0; i < includeScripts.length; i++) {
		var scriptFile = new File(includeScripts[i]);
		if (scriptFile.open()) {
			eval(scriptFile.read());
			scriptFile.close();
		}
		else {
			readyState = false;
			alert(NAME_STR + ":\n\nUnable to open included file: \"" + includeScripts[i] + "\".");
		}
	}

	// test for socket (and FS) privelages
	if (app.preferences.getPrefAsLong("Main Pref Section", "Pref_SCRIPTING_FILE_NETWORK_SECURITY") != 1) {
		readyState = false;
		alert (NAME_STR + ":\n\nThis script requires the scripting security preference to be set.\n\n" +
			"Go to the \"General\" panel of your application preferences,\n" +
			"and make sure that \"Allow Scripts to Write Files and Access Network\" is checked.");
	}

	if (readyState) {
		var optUName, optUPass, optAnon, optHost, optPort, optSave, optPath;

		// UI
		function buildDialog() {
			var dlg = new Window("dialog", NAME_STR, [100, 100, 470, 500]);

			var msg = dlg.addUI("statictext", /* [0, 0, 300, 2 * UILM_D_TEXT_HEIGHT],*/ "Items currently enabled in the render queue will be\nrendered and uploaded to the specified FTP server.");
			msg.justify = "center";
//			msg.dontLayout = true;

			var optsPnl = dlg.addUI("panel", "FTP Options");
			var addrLbl = optsPnl.add("statictext", [0, 0, 130, UILM_D_TEXT_HEIGHT], "Server Name or Address:");
			addrLbl.dontLayout = true;
			optHost = optsPnl.addUI("edittext");
//			var portLbl = optsPnl.add("statictext", [0, 0, 30, UILM_D_TEXT_HEIGHT], " Port:");
//			portLbl.dontLayout = true;
//			optPort = optsPnl.add("edittext", [0, 0, 30, UILM_D_TEXT_HEIGHT]);
//			optHost = optsPnl.addUI("labeledittext", "Server Name or Address:");
			optPort = optsPnl.addUI("labeledittext", "Port:");
			optPort.layoutFill = UILM_FILL_EXPAND;
//			optPort.dontLayout = true;
// 1.3			optsPnl.addUI("statictext", "Remote Path:");
// 1.3			optPath = optsPnl.addUI("edittext");
			optPath = optsPnl.addUI("labeledittext", "Remote Path:");
			optPath.layoutFill = UILM_FILL_EXPAND;
			optAnon = optsPnl.addUI("checkbox", "Anonymous Login");
// 1.3			optsPnl.addUI("statictext", "User Name:");
// 1.3			optUName = optsPnl.addUI("edittext");
			optUName = optsPnl.addUI("labeledittext", "User Name:");
			optUName.layoutFill = UILM_FILL_EXPAND;
// 1.3			optsPnl.addUI("statictext", "Password:");
// 1.3			optUPass = optsPnl.addUI("edittext", "", {noecho: "true"});
			optUPass = optsPnl.addUI("labeledittext", "Password:", {noecho: "true"});
			optUPass.layoutFill = UILM_FILL_EXPAND;
			optSave = optsPnl.addUI("checkbox", "Remember Settings");

			dlg.addUI("button", "Cancel", {name: "cancel"});
			dlg.addUI("button", "Render and FTP", {name: "ok"});

			// handlers
			optAnon.onClick = function () {
				optUName.enabled = !this.value;
				optUPass.enabled = !this.value;
			};

			// defaults
			if (app.settings.haveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_HOST)) {
				var hoststr = app.settings.getSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_HOST);
				var hoststr = hoststr.split(":");
				optHost.text = hoststr[0];
				optPort.text = hoststr[1];
			}
			if (!optPort.text) {
				optPort.text = FTP_DFLT_PORT;
			}

			if (app.settings.haveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_ANON)) {
				optAnon.value = (app.settings.getSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_ANON) == "1");
			}

			if (app.settings.haveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_PATH)) {
				optPath.text = app.settings.getSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_PATH);
			}

			if (app.settings.haveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_UNAME)) {
				optUName.text = app.settings.getSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_UNAME);
			}

			if (app.settings.haveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_UPASS)) {
				optUPass.text = app.settings.getSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_UPASS);
			}

			if (optAnon.value) {
				optUName.enabled = false;
				optUPass.enabled = false;
			}

			layoutUI(dlg);
			dlg.center();
			return dlg;
		}

		// functions
		function savePrefs() {
			if (optSave.value) {
				app.settings.saveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_HOST, optHost.text + ":" + optPort.text);
				app.settings.saveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_PATH, optPath.text);
				app.settings.saveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_ANON, ((optAnon.value) ? 1 : 0));
				app.settings.saveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_UNAME, optUName.text);
				app.settings.saveSetting(FTP_SETTINGS_CAT, FTP_SETTINGS_UPASS, optUPass.text);
			}
		}

		function FTPFiles(fileList) {
			var ftp = new FTPConnection(false);
			try {
				ftp.setEncodingBinary();
				if (optAnon.value) {
					if (!ftp.openAnonymous(optHost.text, optPort.text)) {
						alert(NAME_STR + ":\n\n" + ftp.error);
						return false;
					}
				}
				else {
					if (!ftp.open(optHost.text, optPort.text, optUName.text, optUPass.text)) {
						alert(NAME_STR + ":\n\n" + ftp.error);
						return false;
					}
				}
				// for each file, upload localpath/name to remotepath/name
				for (var i = 0; i < fileList.length; i++) {
					if (!ftp.put(fileList[i], optPath.text + fileList[i].name)) {
						alert(NAME_STR + ":\n\n" + ftp.error);
					}
				}
			}
			catch (e) {
				alert(NAME_STR + ":\n\n" + e.toString() + "\n" + ftp.error);
				return false;
			}
			finally {
				ftp.close();
			}
			return true;
		}

		//////// main ////////
		function main() {
			var proj = app.project;
			var renderq;
			var renderFiles = new Array();

			if (!proj) {
				alert(NAME_STR + ":\n\nPlease open a project in order to render.");
				return false;
			}

			// check for 1+ queued render items
			// build a list of rendered items and their file names/paths
			renderq = proj.renderQueue;
			for (var i = 1; i <= renderq.numItems; i++) {
				if (renderq.item(i).status == RQItemStatus.QUEUED) {
					for (var j = 1; j <= renderq.item(i).numOutputModules; j++) {
						renderFiles[renderFiles.length] = renderq.item(i).outputModule(j).file;
					}
				}
			}

			if (renderFiles.length == 0) {
				alert(NAME_STR + ":\n\nNo render items are currently queued.");
				return false;
			}

			// get settings
			var myWin = buildDialog();
			if (myWin.show() != 1) {
				return false;
			}

			// make sure the path has forward slashes and ends in a slash
			optPath.text = optPath.text.replace("\\", "/");
			if (optPath.text.charAt(optPath.text.length - 1) != "/") {
				optPath.text += "/";
			}

			savePrefs();

			// render
			renderq.render();

			// FTP
			if (FTPFiles(renderFiles)) {
				writeLn("Rendered items successfully transfered to server.");
				return true;
			}

			return false;
		}

		main();
	}
}
